@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2024 OARC, Inc.
+ * Copyright 2019-2026 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -52,6 +52,9 @@
 #include <openssl/ssl.h>
 #include <openssl/conf.h>
 #include <openssl/err.h>
+#ifdef HAVE_JSON_C
+#include <json-c/json.h>
+#endif
 
 #define HISTOGRAM_SIGBITS 5 /* about 3 % latency precision */
 
@@ -103,6 +106,9 @@ typedef struct {
     bool latency_histogram;
 #endif
     int qps_threshold_wait;
+#ifdef HAVE_JSON_C
+    bool json;
+#endif
 } config_t;
 
 typedef struct {
@@ -120,6 +126,7 @@ typedef struct {
     uint64_t num_interrupted;
     uint64_t num_timedout;
     uint64_t num_completed;
+    uint64_t num_unexpected;
 
     uint64_t total_request_size;
     uint64_t total_response_size;
@@ -218,12 +225,54 @@ print_initial_status(const config_t* config)
     char   buf[255], ct[32];
     int    i;
 
+    perf_sockaddr_format(&config->server_addr, buf, sizeof(buf));
+    now = time(NULL);
+
+#ifdef HAVE_JSON_C
+    if (config->json) {
+        int                 ret   = 0;
+        struct json_object* start = json_object_new_object();
+
+        ret |= json_object_object_add(start, "version", json_object_new_string(PACKAGE_VERSION));
+
+        struct json_object* command_line = json_object_new_array();
+        ret |= json_object_array_add(command_line, json_object_new_string(progname));
+        for (i = 1; i < config->argc; i++)
+            ret |= json_object_array_add(command_line, json_object_new_string(config->argv[i]));
+        ret |= json_object_object_add(start, "command_line", command_line);
+
+        ret |= json_object_object_add(start, config->updates ? "updates" : "queries", json_object_new_boolean(1));
+        ret |= json_object_object_add(start, "server_addr", json_object_new_string(buf));
+        ret |= json_object_object_add(start, "server_port", json_object_new_int(perf_sockaddr_port(&config->server_addr)));
+
+        ctime_r(&now, ct);
+        char* nl = strchr(ct, '\n');
+        if (nl)
+            *nl = 0; // Remove added new-line by ctime_r()
+        ret |= json_object_object_add(start, "started", json_object_new_string(ct));
+
+        if (config->timelimit)
+            ret |= json_object_object_add(start, "timelimit", json_object_new_double(config->timelimit / MILLION));
+        if (config->maxruns)
+            ret |= json_object_object_add(start, "maxruns", json_object_new_int(config->maxruns));
+
+        struct json_object* json = json_object_new_object();
+        ret |= json_object_object_add(json, "start", start);
+        if (!ret) {
+            printf("%s\n", json_object_to_json_string_ext(json, JSON_C_TO_STRING_NOSLASHESCAPE));
+        } else {
+            perf_log_fatal("unable to construct JSON output");
+        }
+        json_object_put(json);
+        return;
+    }
+#endif
+
     printf("[Status] Command line: %s", progname);
     for (i = 1; i < config->argc; i++)
         printf(" %s", config->argv[i]);
     printf("\n");
 
-    perf_sockaddr_format(&config->server_addr, buf, sizeof(buf));
     if (perf_sockaddr_isinet6(&config->server_addr)) {
         printf("[Status] Sending %s (to [%s]:%d)\n",
             config->updates ? "updates" : "queries", buf, perf_sockaddr_port(&config->server_addr));
@@ -232,7 +281,6 @@ print_initial_status(const config_t* config)
             config->updates ? "updates" : "queries", buf, perf_sockaddr_port(&config->server_addr));
     }
 
-    now = time(NULL);
     printf("[Status] Started at: %s", ctime_r(&now, ct));
 
     printf("[Status] Stopping after ");
@@ -260,6 +308,26 @@ print_final_status(const config_t* config)
     else
         reason = "time limit";
 
+#ifdef HAVE_JSON_C
+    if (config->json) {
+        int ret = 0;
+
+        struct json_object* stop = json_object_new_object();
+
+        ret |= json_object_object_add(stop, "reason", json_object_new_string(reason));
+
+        struct json_object* json = json_object_new_object();
+        ret |= json_object_object_add(json, "stop", stop);
+        if (!ret) {
+            printf("%s\n", json_object_to_json_string_ext(json, JSON_C_TO_STRING_NOSLASHESCAPE));
+        } else {
+            perf_log_fatal("unable to construct JSON output");
+        }
+        json_object_put(json);
+        return;
+    }
+#endif
+
     printf("[Status] Testing complete (%s)\n", reason);
     printf("\n");
 }
@@ -285,6 +353,7 @@ diff_stats(const config_t* config, stats_t* last, stats_t* now, stats_t* diff)
     diff->num_interrupted = now->num_interrupted - last->num_interrupted;
     diff->num_timedout    = now->num_timedout - last->num_timedout;
     diff->num_completed   = now->num_completed - last->num_completed;
+    diff->num_unexpected  = now->num_unexpected - last->num_unexpected;
 
     diff->total_request_size  = now->total_request_size - last->total_request_size;
     diff->total_response_size = now->total_response_size - last->total_response_size;
@@ -346,6 +415,30 @@ print_histogram(hg64* histogram, const char* const desc)
             pcount);
     };
 }
+
+#ifdef HAVE_JSON_C
+static struct json_object*
+histogram2json(hg64* histogram)
+{
+    struct json_object* hg_array = json_object_new_array();
+
+    uint64_t pmin, pmax, pcount;
+    for (unsigned key = 0;
+         hg64_get(histogram, key, &pmin, &pmax, &pcount) == true;
+         key = hg64_next(histogram, key)) {
+        if (pcount == 0)
+            continue;
+
+        struct json_object* hg_ent = json_object_new_array();
+        json_object_array_add(hg_ent, json_object_new_double((double)pmin / MILLION));
+        json_object_array_add(hg_ent, json_object_new_double((double)pmax / MILLION));
+        json_object_array_add(hg_ent, json_object_new_uint64(pcount));
+        json_object_array_add(hg_array, hg_ent);
+    }
+
+    return hg_array;
+}
+#endif
 #endif
 
 /*
@@ -368,6 +461,92 @@ print_statistics(const config_t* config, const times_t* times, stats_t* stats, u
     else
         run_time = times->end_time - times->start_time;
 
+#ifdef HAVE_JSON_C
+    if (config->json) {
+        int ret = 0;
+
+        struct json_object* statistics = json_object_new_object();
+
+        if (now)
+            ret |= json_object_object_add(statistics, "interval", json_object_new_boolean(1));
+
+        ret |= json_object_object_add(statistics, "sent", json_object_new_uint64(stats->num_sent));
+        ret |= json_object_object_add(statistics, "completed", json_object_new_uint64(stats->num_completed));
+        ret |= json_object_object_add(statistics, "lost", json_object_new_uint64(stats->num_timedout));
+        if (stats->num_interrupted > 0)
+            ret |= json_object_object_add(statistics, "interrupted", json_object_new_uint64(stats->num_interrupted));
+        if (stats->num_unexpected > 0)
+            ret |= json_object_object_add(statistics, "unexpected", json_object_new_uint64(stats->num_unexpected));
+
+        struct json_object* rcodes = json_object_new_object();
+        for (i = 0; i < DNSPERF_STATS_RCODECOUNTS; i++) {
+            if (stats->rcodecounts[i] == 0)
+                continue;
+            ret |= json_object_object_add(rcodes, perf_dns_rcode_strings[i], json_object_new_uint64(stats->rcodecounts[i]));
+        }
+
+        ret |= json_object_object_add(statistics, "request_avg_pktsize", json_object_new_uint64((unsigned int)PERF_SAFE_DIV(stats->total_request_size, stats->num_sent)));
+        ret |= json_object_object_add(statistics, "response_avg_pktsize", json_object_new_uint64((unsigned int)PERF_SAFE_DIV(stats->total_response_size, stats->num_completed)));
+
+        ret |= json_object_object_add(statistics, "run_time", json_object_new_double(run_time / MILLION));
+        ret |= json_object_object_add(statistics, "qps", json_object_new_double(PERF_SAFE_DIV(stats->num_completed, (((double)(now ? interval_time : run_time) / MILLION)))));
+
+        {
+            struct json_object* latency = json_object_new_object();
+            ret |= json_object_object_add(latency, "avg", json_object_new_double((double)PERF_SAFE_DIV(stats->latency_sum, stats->num_completed) / MILLION));
+            if (!now) {
+                ret |= json_object_object_add(latency, "min", json_object_new_double((double)stats->latency_min / MILLION));
+                ret |= json_object_object_add(latency, "max", json_object_new_double((double)stats->latency_max / MILLION));
+            }
+
+            if (stats->num_completed > 1) {
+                ret |= json_object_object_add(latency, "stddev", json_object_new_double(stddev(stats->latency_sum_squares, stats->latency_sum, stats->num_completed) / MILLION));
+#ifdef USE_HISTOGRAMS
+                if (config->latency_histogram)
+                    ret |= json_object_object_add(latency, "histogram", histogram2json(stats->latency));
+#endif
+            }
+
+            ret |= json_object_object_add(statistics, "latency", latency);
+        }
+
+        if (stats->num_conn_completed || stats->num_conn_attempts) {
+            struct json_object* connection = json_object_new_object();
+            ret |= json_object_object_add(connection, "attempts", json_object_new_uint64(stats->num_conn_attempts));
+            ret |= json_object_object_add(connection, "completed", json_object_new_uint64(stats->num_conn_completed));
+
+            struct json_object* latency = json_object_new_object();
+            ret |= json_object_object_add(latency, "avg", json_object_new_double((double)PERF_SAFE_DIV(stats->conn_latency_sum, stats->num_conn_completed) / MILLION));
+            if (!now) {
+                ret |= json_object_object_add(latency, "min", json_object_new_double((double)stats->conn_latency_min / MILLION));
+                ret |= json_object_object_add(latency, "max", json_object_new_double((double)stats->conn_latency_max / MILLION));
+            }
+
+            if (stats->num_conn_completed > 1) {
+                ret |= json_object_object_add(latency, "stddev", json_object_new_double(stddev(stats->conn_latency_sum_squares, stats->conn_latency_sum, stats->num_conn_completed) / MILLION));
+#ifdef USE_HISTOGRAMS
+                if (config->latency_histogram)
+                    ret |= json_object_object_add(latency, "histogram", histogram2json(stats->conn_latency));
+#endif
+            }
+            ret |= json_object_object_add(connection, "latency", latency);
+
+            ret |= json_object_object_add(statistics, "connection", connection);
+        }
+
+        struct json_object* json = json_object_new_object();
+        ret |= json_object_object_add(json, "statistics", statistics);
+        if (!ret) {
+            printf("%s\n", json_object_to_json_string_ext(json, JSON_C_TO_STRING_NOSLASHESCAPE));
+        } else {
+            perf_log_fatal("unable to construct JSON output");
+        }
+        json_object_put(json);
+        fflush(stdout);
+        return;
+    }
+#endif
+
     printf("%sStatistics:\n\n", now ? "Interval " : "");
 
     printf("  %s sent:         %" PRIu64 "\n",
@@ -382,6 +561,11 @@ print_statistics(const config_t* config, const times_t* times, stats_t* stats, u
         printf("  %s interrupted:  %" PRIu64 " (%.2lf%%)\n",
             units, stats->num_interrupted,
             PERF_SAFE_DIV(100.0 * stats->num_interrupted, stats->num_sent));
+    if (stats->num_unexpected > 0)
+        printf("  Unexpected IDs:       %" PRIu64 " (%.2lf%%)\n",
+            stats->num_unexpected,
+            PERF_SAFE_DIV(100.0 * stats->num_unexpected, stats->num_sent));
+
     printf("\n");
 
     printf("  Response codes:       ");
@@ -466,7 +650,7 @@ print_statistics(const config_t* config, const times_t* times, stats_t* stats, u
             stddev(stats->conn_latency_sum_squares, stats->conn_latency_sum, stats->num_conn_completed) / MILLION);
 #ifdef USE_HISTOGRAMS
         if (config->latency_histogram)
-            print_histogram(stats->latency, "connection count");
+            print_histogram(stats->conn_latency, "connection count");
 #endif
     }
 
@@ -506,6 +690,7 @@ sum_stats(const config_t* config, stats_t* total)
         total->num_interrupted += stats->num_interrupted;
         total->num_timedout += stats->num_timedout;
         total->num_completed += stats->num_completed;
+        total->num_unexpected += stats->num_unexpected;
 
         total->total_request_size += stats->total_request_size;
         total->total_response_size += stats->total_response_size;
@@ -665,6 +850,11 @@ setup(int argc, char** argv, config_t* config)
     perf_opt_add('v', perf_opt_boolean, NULL,
         "verbose: report each query and additional information to stdout",
         NULL, &config->verbose);
+#ifdef HAVE_JSON_C
+    perf_opt_add('j', perf_opt_boolean, NULL,
+        "use JSON output for run information and statistics",
+        NULL, &config->json);
+#endif
     perf_long_opt_add("doh-uri", perf_opt_string, "doh_uri",
         "the URI to use for DNS-over-HTTPS", DEFAULT_DOH_URI, &doh_uri);
     perf_long_opt_add("doh-method", perf_opt_string, "doh_method",
@@ -774,7 +964,7 @@ setup(int argc, char** argv, config_t* config)
      * We also can't run more threads than clients.
      */
     if (config->threads > config->clients) {
-        perf_log_warning("requested number of threads (-T %u) exceeds number of clients (-c %u), lowering number of threads\n", config->threads, config->clients);
+        perf_log_warning("requested number of threads (-T %u) exceeds number of clients (-c %u), lowering number of threads", config->threads, config->clients);
         config->threads = config->clients;
     }
 
@@ -791,6 +981,15 @@ setup(int argc, char** argv, config_t* config)
             perf_log_fatal("Unable to measure nanosleep(): %s", perf_strerror_r(errno, __s, sizeof(__s)));
         }
     }
+
+#ifdef HAVE_JSON_C
+    if (config->json && config->verbose) {
+        perf_log_fatal("Verbose output (-v) is currently not possible while using JSON output (-j)");
+    }
+    if (config->json && log_stdout) {
+        perf_log_fatal("Logging warnings and errors to stdout (-W) is currently not possible while using JSON output (-j)");
+    }
+#endif
 }
 
 static void
@@ -1337,6 +1536,7 @@ do_recv(void* arg)
                                      "id: %u",
                         recvd[i].qid);
                 }
+                stats->num_unexpected++;
                 continue;
             }
             latency = recvd[i].when - recvd[i].sent;
@@ -1420,9 +1620,30 @@ do_interval_stats(void* arg)
             print_statistics(tinfo->config, tinfo->times, &diff, now, interval_time);
         } else {
             qps = (total.num_completed - last.num_completed) / (((double)interval_time) / MILLION);
-            perf_log_printf("%u.%06u: %.6lf",
-                (unsigned int)(now / MILLION),
-                (unsigned int)(now % MILLION), qps);
+#ifdef HAVE_JSON_C
+            if (tinfo->config->json) {
+                int ret = 0;
+
+                struct json_object* rate = json_object_new_object();
+
+                ret |= json_object_object_add(rate, "timestamp_usec", json_object_new_uint64(now));
+                ret |= json_object_object_add(rate, "qps", json_object_new_double(qps));
+
+                struct json_object* json = json_object_new_object();
+                ret |= json_object_object_add(json, "rate", rate);
+                if (!ret) {
+                    printf("%s\n", json_object_to_json_string_ext(json, JSON_C_TO_STRING_NOSLASHESCAPE));
+                } else {
+                    perf_log_fatal("unable to construct JSON output");
+                }
+                json_object_put(json);
+            } else
+#endif
+            {
+                perf_log_printf("%u.%06u: %.6lf",
+                    (unsigned int)(now / MILLION),
+                    (unsigned int)(now % MILLION), qps);
+            }
         }
 
         last_interval_time = now;
@@ -1560,12 +1781,12 @@ threadinfo_init(threadinfo_t* tinfo, const config_t* config,
      * We can't have more than 64k outstanding queries per thread.
      */
     if (tinfo->max_outstanding > NQIDS) {
-        perf_log_warning("requested number of outstanding queries (-q %u) per single thread (-T) exceeds built-in maximum %u, adjusting\n", tinfo->max_outstanding, NQIDS);
+        perf_log_warning("requested number of outstanding queries (-q %u) per single thread (-T) exceeds built-in maximum %u, adjusting", tinfo->max_outstanding, NQIDS);
         tinfo->max_outstanding = NQIDS;
     }
 
     if (tinfo->nsocks > MAX_SOCKETS) {
-        perf_log_warning("requested number of clients (-c %u) per thread (-T) exceeds built-in maximum %u, adjusting\n", tinfo->nsocks, MAX_SOCKETS);
+        perf_log_warning("requested number of clients (-c %u) per thread (-T) exceeds built-in maximum %u, adjusting", tinfo->nsocks, MAX_SOCKETS);
         tinfo->nsocks = MAX_SOCKETS;
     }
 
@@ -1632,9 +1853,6 @@ int main(int argc, char** argv)
     perf_result_t          result;
     struct perf_net_socket sock = { .mode = sock_pipe };
 
-    printf("DNS Performance Testing Tool\n"
-           "Version " PACKAGE_VERSION "\n\n");
-
     (void)SSL_library_init();
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
     SSL_load_error_strings();
@@ -1643,6 +1861,12 @@ int main(int argc, char** argv)
 
     setup(argc, argv, &config);
 
+#ifdef HAVE_JSON_C
+    if (!config.json)
+#endif
+        printf("DNS Performance Testing Tool\n"
+               "Version " PACKAGE_VERSION "\n\n");
+
     if (pipe(threadpipe) < 0 || pipe(mainpipe) < 0 || pipe(intrpipe) < 0)
         perf_log_fatal("creating pipe");
 
@@ -1711,7 +1935,11 @@ int main(int argc, char** argv)
 
     sum_stats(&config, &total_stats);
     print_statistics(&config, &times, &total_stats, 0, 0);
-    perf_net_stats_print(config.mode);
+#ifdef HAVE_JSON_C
+    perf_net_stats_print(config.mode, config.json);
+#else
+    perf_net_stats_print(config.mode, false);
+#endif
 
     cleanup(&config);
 #if OPENSSL_VERSION_NUMBER < 0x10100000L
