@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2024 OARC, Inc.
+ * Copyright 2019-2026 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -179,10 +179,10 @@ void perf_net_doh_set_max_concurrent_streams(size_t);
 
 void perf_net_stats_init(enum perf_net_mode);
 void perf_net_stats_compile(enum perf_net_mode, struct perf_net_socket*);
-void perf_net_stats_print(enum perf_net_mode);
+void perf_net_stats_print(enum perf_net_mode, bool json);
 void perf_net_doh_stats_init();
 void perf_net_doh_stats_compile(struct perf_net_socket*);
-void perf_net_doh_stats_print();
+void perf_net_doh_stats_print(bool json);
 
 extern const char* perf_net_tls_sni;
 
