@@ -1,5 +1,5 @@
 /*
- * Copyright 2019-2024 OARC, Inc.
+ * Copyright 2019-2026 OARC, Inc.
  * Copyright 2017-2018 Akamai Technologies
  * Copyright 2006-2016 Nominum, Inc.
  * All rights reserved.
@@ -46,6 +46,9 @@
 #include <nghttp2/nghttp2.h>
 #include <openssl/bio.h>
 #include <openssl/evp.h>
+#ifdef HAVE_JSON_C
+#include <json-c/json.h>
+#endif
 
 #define DNS_GET_REQUEST_VAR "?dns="
 #define DNS_MSG_MAX_SIZE 65535
@@ -1047,8 +1050,40 @@ void perf_net_doh_stats_compile(struct perf_net_socket* sock)
     doh_stats.unknown_code += self->stats.unknown_code;
 }
 
-void perf_net_doh_stats_print()
+void perf_net_doh_stats_print(bool json)
 {
+#ifdef HAVE_JSON_C
+    if (json) {
+        int ret = 0;
+
+        struct json_object* dns_over_https = json_object_new_object();
+
+        struct json_object* status_codes = json_object_new_object();
+        int                 i;
+        char                code[16];
+        for (i = 0; i < (sizeof(doh_stats.code) / sizeof(doh_stats.code[0])); i++) {
+            if (doh_stats.code[i]) {
+                snprintf(code, sizeof(code), "%d", i + 100);
+                ret |= json_object_object_add(status_codes, code, json_object_new_uint64(doh_stats.code[i]));
+            }
+        }
+        if (doh_stats.unknown_code) {
+            ret |= json_object_object_add(status_codes, "unknown", json_object_new_uint64(doh_stats.unknown_code));
+        }
+        ret |= json_object_object_add(dns_over_https, "status_codes", status_codes);
+
+        struct json_object* json = json_object_new_object();
+        ret |= json_object_object_add(json, "dns_over_https", dns_over_https);
+        if (!ret) {
+            printf("%s\n", json_object_to_json_string_ext(json, JSON_C_TO_STRING_NOSLASHESCAPE));
+        } else {
+            perf_log_fatal("unable to construct JSON output");
+        }
+        json_object_put(json);
+        return;
+    }
+#endif
+
     printf("DNS-over-HTTPS statistics:\n\n");
 
     printf("  HTTP/2 return codes:  ");
